import { isAsyncIterable, isIterable } from './utils/iterator.js';
export function flatMap(fn) {
    return function flatMapFn(data) {
        if (data == null)
            return;
        // Handle the async version
        if (isAsyncIterable(data) || data instanceof Promise) {
            return {
                async *[Symbol.asyncIterator]() {
                    let stream = data instanceof Promise ? await data : data;
                    let i = 0;
                    for await (let datum of stream) {
                        let result = fn(datum, i++);
                        if (isAsyncIterable(result) || result instanceof Promise) {
                            let stream = result instanceof Promise ? await result : result;
                            yield* stream;
                        }
                        else if (isIterable(result)) {
                            yield* result;
                        }
                        else {
                            yield result;
                        }
                    }
                },
            };
        }
        // Handle the sync version
        return {
            *[Symbol.iterator]() {
                let i = 0;
                for (let datum of data) {
                    let result = fn(datum, i++);
                    if (isIterable(result)) {
                        yield* result;
                    }
                    else {
                        yield result;
                    }
                }
            },
        };
    };
}
